﻿Imports Newtonsoft.Json

Public Class RedBlackTree(Of TKey As IComparable(Of TKey), TValue)
    Private NotInheritable Class Node
        Public Property Key As TKey
        Public Property Value As TValue
        Public Property IsRed As Boolean
        Public Property Left As Node
        Public Property Right As Node
    End Class

    <JsonProperty>
    Private root As Node

    <JsonProperty>
    Public ReadOnly Property Count As Integer
        Get
            Return _count
        End Get
    End Property
    Private _count As Integer

    Public Sub Add(key As TKey, value As TValue)

        root = AddTo(root, key, value)
        root.IsRed = False

        _count += 1

    End Sub

    Private Function AddTo(node As Node, key As TKey, value As TValue) As Node
        If node Is Nothing Then
            Return New Node() With {.Key = key, .Value = value, .IsRed = True}
        End If

        Dim cmp = key.CompareTo(node.Key)
        If (cmp < 0) Then
            node.Left = AddTo(node.Left, key, value)
        ElseIf (cmp > 0) Then
            node.Right = AddTo(node.Right, key, value)
        Else
            node.Value = value
        End If

        If IsRed(node.Right) And Not IsRed(node.Left) Then ' Regel für Linkslastigkeit
            node = RotateLeft(node)
        End If

        If IsRed(node.Left) And IsRed(node.Left.Left) Then ' Keine aufeinanderfolgenden roten Knoten
            node = RotateRight(node)
        End If

        If IsRed(node.Left) And IsRed(node.Right) Then    ' keine zwei roten Kindknoten
            FlipColors(node)
        End If

        Return node
    End Function

    Private Shared Function RotateLeft(node As Node) As Node
        Dim newParent = node.Right

        node.Right = newParent.Left
        newParent.Left = node
        newParent.IsRed = newParent.Left.IsRed
        newParent.Left.IsRed = True

        Return newParent
    End Function

    Private Shared Function RotateRight(node As Node) As Node
        Dim newParent = node.Left

        node.Left = newParent.Right
        newParent.Right = node
        newParent.IsRed = newParent.Right.IsRed
        newParent.Right.IsRed = True

        Return newParent
    End Function

    Private Shared Function IsRed(node As Node) As Boolean
        If node Is Nothing Then  ' Nothing-Verweise stellen schwarze Blätter dar
            Return False
        End If

        Return node.IsRed
    End Function

    Private Shared Sub FlipColors(node As Node)
        node.IsRed = Not node.IsRed
        node.Left.IsRed = Not node.IsRed
        node.Right.IsRed = Not node.IsRed
    End Sub

    Public Function Contains(key As TKey)

        Dim node As Node = root
        While node IsNot Nothing
            Dim c As Integer = key.CompareTo(node.Key)

            If c = 0 Then
                Return True
            ElseIf c < 0 Then
                node = node.Left
            Else
                node = node.Right
            End If
        End While
        Return False
    End Function

    Default Public ReadOnly Property Item(key As TKey) As TValue
        Get
            Dim node As Node = root
            While node IsNot Nothing
                Dim c As Integer = key.CompareTo(node.Key)

                If c = 0 Then
                    Return node.Value
                ElseIf c < 0 Then
                    node = node.Left
                Else
                    node = node.Right
                End If
            End While
            Throw New ArgumentException("Schlüssel nicht vorhanden")
        End Get
    End Property
    Public Overrides Function ToString() As String
        Dim s As String = ""
        Dim level As Integer = 0

        Traverse(root, level, s)

        Return s
    End Function

    Private Sub Traverse(node As Node, level As Integer, ByRef s As String)
        If node Is Nothing Then
            Return
        End If

        Dim color As String = If(node.IsRed = True, "red", "black")
        s += "".PadLeft(level, " "c) + $"{node.Key} ({color})" + vbLf
        Traverse(node.Left, level + 2, s)
        Traverse(node.Right, level + 2, s)
    End Sub
End Class
